@@ -174,7 +174,6 @@ def data_science_engine_factory(
     password = vars_dict.get("password")
 
     if password:
-
         conn_string = snowflake_URL(
             user=vars_dict.get("user"),
             password=password,
@@ -290,42 +289,46 @@ def snowflake_stage_load_copy_remove(
     file_format_options: str = "",
 ) -> None:
     """
-    Upload file to stage, copy to table, remove file from stage on Snowflake
+    This function performs remove+upload+copy+remove:
+    - Remove file from Snowflake internal stage if it currently exists from prev process.
+    - Upload/put file to internal stage
+    - Copy file from internal stage to target table
+    - Finally, remove file from internal stage
     """
 
+    file_name = os.path.basename(file)
+    full_stage_file_path = f"{stage}/{file_name}.gz"
+    remove_query = f"remove @{full_stage_file_path}"
     put_query = f"put 'file://{file}' @{stage} auto_compress=true;"
 
     if type == "json":
         copy_query = f"""copy into {table_path} (jsontext)
-                         from @{stage}
+                         from @{full_stage_file_path}
                          file_format=(type='{type}'),
-                         on_error='{on_error}';"""
+                         on_error='{on_error}';
+                         """
 
-        remove_query = f"remove @{stage} pattern='.*.{type}.gz'"
     else:
-        file_name = os.path.basename(file)
-        file_pattern = f".*{file_name}.gz"
-        copy_query = f"""copy into {table_path} 
-                         from @{stage}
+        copy_query = f"""copy into {table_path}
+                         from @{full_stage_file_path}
                          file_format=(type='{type}' {file_format_options}),
-                         on_error='{on_error}'
-                         pattern='{file_pattern}';
+                         on_error='{on_error}';
                         """
 
-        remove_query = f"remove @{stage} pattern='{file_pattern}'"
-
     logging.basicConfig(stream=sys.stdout, level=20)
 
     try:
         connection = engine.connect()
 
-        logging.info(f"Clearing {type} files from stage.")
-        remove = connection.execute(remove_query)
-        logging.info(f"Query successfully run")
+        logging.info(
+            f"Removing file from internal stage with full_stage_file_path: {full_stage_file_path}"
+        )
+        connection.execute(remove_query)
+        logging.info("Query successfully run")
 
         logging.info("Writing to Snowflake.")
-        results = connection.execute(put_query)
-        logging.info(f"Query successfully run")
+        connection.execute(put_query)
+        logging.info("Query successfully run")
     finally:
         connection.close()
         engine.dispose()
@@ -334,12 +337,12 @@ def snowflake_stage_load_copy_remove(
         connection = engine.connect()
 
         logging.info(f"Copying to Table {table_path}.")
-        copy_results = connection.execute(copy_query)
-        logging.info(f"Query successfully run")
+        connection.execute(copy_query)
+        logging.info("Query successfully run")
 
         logging.info(f"Removing {file} from stage.")
-        remove = connection.execute(remove_query)
-        logging.info(f"Query successfully run")
+        connection.execute(remove_query)
+        logging.info("Query successfully run")
     finally:
         connection.close()
         engine.dispose()
@@ -393,12 +396,75 @@ def write_to_gsheets(
     logging.basicConfig(stream=sys.stdout, level=20)
 
     try:
-        logging.info(f"Writing data to sheet {sheet_name}...")
+        logging.info(f"Trying to create a sheet called {sheet_name}...")
         sheet.add_worksheet(sheet_name)
-        logging.info(f"Writing completed.")
-    except:
-        pass
-    wks_write = sheet.worksheet_by_title(sheet_name)
-    wks_write.clear("A1", None, "*")
-    wks_write.set_dataframe(data_df, (1, 1), encoding="utf-8", fit=True)
-    wks_write.frozen_rows = 1
+        logging.info("New tab created.")
+
+    except Exception as exc:
+        # A sheet with that name might already exist
+        logging.error(exc)
+
+    try:
+        wks_write = sheet.worksheet_by_title(sheet_name)
+        wks_write.clear("A1", None, "*")
+        wks_write.set_dataframe(data_df, (1, 1), encoding="utf-8", fit=True)
+        wks_write.frozen_rows = 1
+
+    except Exception as exc:
+        logging.error(exc)
+        logging.error("GSheet could not be written.")
+
+
+def read_from_gsheets(
+    spreadsheet_id: str,
+    sheet_name: str,
+    service_account_credentials: str = "GSHEETS_SERVICE_ACCOUNT_CREDENTIALS",
+) -> pd.DataFrame:
+    """
+    Gets an spreadsheet id and tab name as a parameter and tries to read the
+    content into a pandas dataframe.
+    """
+    gsheets = pygsheets.authorize(service_account_env_var=service_account_credentials)
+    sheet = gsheets.open_by_key(spreadsheet_id)
+
+    logging.basicConfig(stream=sys.stdout, level=20)
+
+    df = None
+
+    try:
+        logging.info(f"Reading data from sheet {sheet_name}...")
+        wks_read = sheet.worksheet_by_title(sheet_name)
+        df = wks_read.get_as_df()
+        logging.info("Read completed.")
+
+    except Exception as exc:
+        logging.error(exc)
+        logging.error("GSheet could not be read.")
+
+    return df
+
+
+def query_from_file(engine: Engine, query_path: str) -> pd.DataFrame:
+    """
+    Execute DB queries from a SQL file.
+    """
+
+    try:
+        logging.info(f"Reading query from file {query_path}...")
+        file = open(query_path, "r")
+        query = file.read()
+        file.close()
+
+    except Exception as exc:
+        logging.error(exc)
+        logging.error("Query could not be read.")
+
+    df = None
+
+    try:
+        df = query_dataframe(engine, query)
+    except Exception as exc:
+        logging.error(exc)
+        logging.error("Query did not execute.")
+
+    return df
